//
// Simple Shoot
// Copyright (C) 2010-2011 Xavier MICHEL (xavier.michel.mx440@gmail.com)
//
// This software is provided as-is, without any express or implied warranty.
// In no event will the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it freely,
// subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented;
//    you must not claim that you wrote the original software.
//    If you use this software in a product, an acknowledgment
//    in the product documentation would be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such,
//    and must not be misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source distribution.
//

#ifndef PARAMETRES_HPP
#define PARAMETRES_HPP

#include "Singleton.hpp"

/*!
    \class Parametres
    \brief Les paramètres de l'application, son activé par exemple
    L'avantage de le faire dans une classe à part c'est que l'on pourrait sauvegarder toutes les préférences et les recharger au démarage du jeu
*/

class Parametres : public Singleton<Parametres>
{

friend class Singleton<Parametres>;

public:
        /*!
            \brief Active ou désactive le son
        */
        static void toogleSound();

        /*!
            \brief Renvoie vrai si le son est activé
        */
        static bool isMute();

        /*!
            \brief Définie le pseudo du joueur
        */
        static void setPseudo(const std::string &pseudo);

        /*!
            \brief Renvoie le nom du joueur
        */
        static std::string getPseudo();

protected:

        Parametres();

        bool m_muet;                //!< Doit-on jouer les sons du jeu ?

        std::string m_pseudoJoueur; //!< Pseudo du joueur
};

#endif // PARAMETRES_HPP
