//
// Simple Shoot
// Copyright (C) 2010-2011 Xavier MICHEL (xavier.michel.mx440@gmail.com)
//
// This software is provided as-is, without any express or implied warranty.
// In no event will the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it freely,
// subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented;
//    you must not claim that you wrote the original software.
//    If you use this software in a product, an acknowledgment
//    in the product documentation would be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such,
//    and must not be misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source distribution.
//

#ifndef SOUND_MANAGER_HPP
#define SOUND_MANAGER_HPP

#include "Singleton.hpp"


/*!
    \struct SoundData
    \brief Les données qui sont attachées à un son
*/
struct SoundData {
    std::string url;            //!< url de la ressource
    bool isLoaded;              //!< La ressource est-elle chargée ?
    sf::SoundBuffer *buffer;    //!< Le buffer du son
    sf::Sound *sound;           //!< Le son en lui même
};

/*!
    \class SoundManager
    \brief Le gestionnaire de sons
    Charge la ressource seulement quand celle-ci est demandée.
*/

class SoundManager : public Singleton<SoundManager>
{

friend class Singleton<SoundManager>;

public :

        /*!
            \brief Demande de jouer un son
            Si la ressource n'est pas encore chargée, elle est chargée à ce moment.
            Si elle est déjà chargée, on jour le son.
        */
        static void playSound(const std::string &r);


protected:

        SoundManager();

        ~SoundManager();

        std::map<std::string, SoundData *> m_ressources;        //!< Les ressources en elle-même
};

#endif // SOUND_MANAGER_HPP

