//
// Simple Shoot
// Copyright (C) 2010-2011 Xavier MICHEL (xavier.michel.mx440@gmail.com)
//
// This software is provided as-is, without any express or implied warranty.
// In no event will the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it freely,
// subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented;
//    you must not claim that you wrote the original software.
//    If you use this software in a product, an acknowledgment
//    in the product documentation would be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such,
//    and must not be misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source distribution.
//

#ifndef JOUEUR_HPP
#define JOUEUR_HPP

#include "constantes.hpp"
#include "Pion.hpp"
#include "ArmeManagerUI.hpp"

class Monde;

/*!
    \class Joueur
    \brief Le vrai joueur, c'est à dire vous !
*/

class Joueur : public Pion  {

public:

    /*!
        \param monde Le monde dans lequel évolue le joueur
        \param position La position initiale du joueur
        \param taillePion La taille du pion du joueur
    */
    Joueur(
            Monde *monde,
            const sf::Vector2f &position,
            const sf::Vector2f &taillePion = sf::Vector2f(16, 24)
    );


    /*!
        \brief Mise à jour du joueur : rien pour l'instant
    */
    void update();

    /*!
        \brief Mise à jour de l'affichage
    */
    void Render(sf::RenderTarget& Target) const;

    /*!
        \brief Ajoute une arme au joueur
    */
    void addWeapon(const std::string &arme);

    /*!
        \brief Enleve une arme au joueur
    */
    void removeWeapon(const std::string &arme);

    /*!
        \brief Selection de l'arme suivante
    */
    void nextWeapon();

    /*!
        \brief Selection de l'arme précédente
    */
    void previousWeapon();

protected :

    ArmeManagerUI m_armeManagerUI;  //!< Représentation graphique des armes que j'ai

};

#endif // JOUEUR_HPP
